App = {
	title: null,

	init: function (w) {
		if (this.title) return;
		this.title = document.documentElement.getAttribute("title");
		User.init();
		Sidebar.init();
		Status.init();
		Schedule.init();
		Calendar.init();
		Body.init();
		window.setTimeout("App.start();", 1);
		},

	start: function () {
		try {
			Subject.init();
			Index.init();
			Diary.init();
			Category.init();
			View.init();
			Journals.init();
			Crypto.init();
			Journals.display();
			}
		catch (e) {alert("App.start " + e.lineNumber + "\n" + e);}
		}
	}


Body = {
	rich: null,
	deck: null,

	blank: null,
	entry: null,
	editmode: null,
	editmenu: null,
	edittool: null,
	front: null,

	viewer: null,
	editor: null,
	nsieditor: null,
	savetimer: null,

	where: null,
	working: false,

	// a textbox lost focus
	blur: function (e) {
		if (this.working) return;
		var t = e.parentNode.parentNode;
		if (t._timer) {
			clearTimeout(t._timer);
			t._timer = null;
			this.save(t);
			}
		},

	clear: function () {
		this.deck.selectedPanel = this.blank;
		},

	// display true = Entry, false = front, default = blank
	display: function (e) {
		this.working = true;
		if (e === true) {
			document.getElementById("entry-name").value = Entry.name;
			document.getElementById("entry-category").value = Entry.category;
			document.getElementById("entry-index").value = Entry.index;
			document.getElementById("entry-created").value = Entry.created;
			document.getElementById("entry-modified").value = Entry.modified;
			document.getElementById("entry-start").value = Entry.start;
			document.getElementById("entry-end").value = Entry.end;

			var bg = Journal.getProperty("paper").match(/(\S+;)/)[1];
			document.getElementById("body-entry").style.backgroundColor = bg;

			if (this.rich) {
				var s = 'background-color:' + bg +
					'font-family:' +
						Journal.getProperty("fontface") + ',' +
						Journal.getProperty("fonttype") + ';' +
					'font-size:' + Journal.getProperty("fontsize") + 'pt;';	
				var frame = (Entry.editmode)? this.editor : this.viewer;
				with (frame.contentDocument) {
					write(
						'<html xmlns="http://www.w3.org/1999/xhtml">' +
						'<head>' +
						'<link rel="stylesheet" type="text/css" href="chrome://journal/skin/entry.css"/>' +
						'</head>' +
						'<body style="' + s + '">' +
						Entry.text +
						'</body>' +
						'</html>');
					close();
					}
				}
			else {
				this.textbox.value = Entry.text;
				this.textbox.readonly = !Entry.editmode;
				}

			if (Entry.editmode) {
				this.editmode.removeAttribute("readonly");
				this.editmenu.setAttribute("checked", "true");
				if (this.rich) {
					with (this.editor) {
						setAttribute("edit", "true");
						this.nsieditor = getEditor(contentWindow);
						contentWindow.focus();
						}
					this.nsieditor.addDocumentStateListener(this);
					}
				else {
					this.textbox.focus();
					}
				this.edittool.checked = true;
				}
			else {
				this.editmode.setAttribute("readonly", "true");
				this.editmenu.removeAttribute("checked");
				this.editor.removeAttribute("edit");
				window.self.focus();
				if (this.nsieditor) {
					this.nsieditor.removeDocumentStateListener(this);
					this.nsieditor = null;
					}
				this.edittool.checked = false;
				}

			if (this.rich) this.modedeck.selectedPanel = frame;
			else this.modedeck.selectedPanel = this.textbox;

			this.deck.selectedPanel = this.entry;
			this._allowEdit(true);
			}
		else if (e === false) {
			this.front.datasources = Journal.ds.URI;
			this.deck.selectedPanel = this.front;
			this._allowEdit(true);
			}
		else {
			this.deck.selectedPanel = this.blank;
			this._allowEdit(false);
			}
		this.working = false;
		},


	// an input element got focus
	focus: function (e) {
		this.where = e;
		},

	init: function () {
		try {
			this.rich = (Components.classes["@mozilla.org/editor/editingsession;1"] != undefined);

			this.deck = document.getElementById("body-deck");
			this.blank = document.getElementById("body-blank");
			this.editmode = document.getElementById("entry-editmode");
			this.editmenu = document.getElementById("edit-editmode");
			this.edittool = document.getElementById("tool-edit");
			this.entry = document.getElementById("body-entry");
			this.front = document.getElementById("body-front");

			this.modedeck = document.getElementById("entry-text");
			this.viewer = document.getElementById("entry-read");
			this.editor = document.getElementById("entry-edit");
			this.textbox = document.getElementById("entry-textbox");
			} catch (e) {alert("Body.init\n" + e);}
		},

	// set edit mode
	mode: function (isedit) {
		try {
			if (isedit == undefined) isedit = this.editmode.hasAttribute("readonly");
			if (!isedit) {
				if (!this.where) this.where = this.editor;
				this.release();
				}
			Entry.editmode = isedit;
			this.display(true);
			}
		catch (e) {alert("Body.mode\n" + e);}
		},

	// nsIDocumentStateListener
	NotifyDocumentCreated: function () {},

	NotifyDocumentStateChanged: function (nowDirty) {
		if (nowDirty) {
			if (this.savetimer) clearTimeout(this.savetimer);
			this.savetimer = setTimeout(this.save, 2000, this.editor);
			this.nsieditor.resetModificationCount();
			}
		},

	NotifyDocumentWillBeDestroyed: function () {},


	print: function () {
		this.editor.contentWindow.print();
		},

	redisplay: function () {
		if (this.deck.selectedPanel == this.entry) this.display(true);
		else if (this.deck.selectedPanel == this.front) this.display(false);
		},


	// release the current displayed entry
	release: function () {
		if (!Entry.editmode) {
			return;
			}
		if (this.where) {
			this.blur(this.where);
			this.save(this.where);
			this.where = null;
			}
		if (this.savetimer) {
			clearTimeout(this.savetimer);
			this.savetimer = null;
			this.save(this.editor);
			}
		},

	// remove the current displayed entry
	remove: function () {
		var n = document.getElementById("entry-name").value;
		if (!User.prompter.confirm(
			self,
			this.entry.getAttribute("deltitle"),
			this.entry.getAttribute("delprompt").replace(/%/, n)
			)) return;
		this.release();
		Entry.remove();
		this.display(null);
		},

	// a textbox or editor timed out
	save: function (t) {
		try {
			switch (t.id) {
				case "entry-name":
					Entry.name = t.value;
					break;
				case "entry-edit":
					if (this.rich) Entry.text = Body.editor.contentDocument.body.innerHTML;
					else Entry.text = Body.textbox.value;
					break;
				}
			}
		catch (e) {alert("Body.save\n" + e);}
		},

	_allowEdit: function (m) {
		with (document.getElementById("edit-command")) {
			if (m) removeAttribute("disabled");
			else setAttribute("disabled", "true");
			}
		}

	}


Calendar = {
	base: null,
	menu: null,
	picker: null,
	splitter: null,
	tool: null,

	dayLong: null,
	dayShort: null,
	monthLong: null,
	monthShort: null,

	init: function () {
		try {
			this.base = document.getElementById("calendar");
			this.menu = document.getElementById("view-calendar");
			this.splitter = document.getElementById("calendar-splitter");
			this.picker = document.getElementById("calendar");
			this.tool = document.getElementById("tool-calendar");

			this.dayLong = this.base.getAttribute("daylong").split(" ");
			this.dayShort = this.base.getAttribute("dayshort").split(" ");
			this.monthLong = this.base.getAttribute("monthlong").split(" ");
			this.monthShort = this.base.getAttribute("monthshort").split(" ");

			this.picker.init();
			} catch (e) {alert("Calendar.init\n" + e);}
		},

	showHide: function (seen) {
		with (this.base) {
			if (seen == !collapsed) return;
			if (seen == undefined) seen = collapsed;
			collapsed = !seen;
			}
		this.splitter.setAttribute("state", ((seen)? "" : "collapsed"));
		this.menu.setAttribute("checked", seen);
		this.tool.checked = !seen;
		Sidebar.showHide(true);
		},

	resized: function () {
		var c = this.base.collapsed;
		this.menu.setAttribute("checked", !c);
		this.tool.checked = c;
		}

	}


Category = {
	command: null,
	panel: null,
	tree: null,

	display: function () {
		this.tree.datasources = Journal.ds.URI;
		Sidebar.display(this.panel);
		View.set("view-category");
		},

	enable: function (enabled) {
		with (this.command) 
			if (enabled) removeAttribute("disabled")
			else setAttribute("disabled", "true");
		},

	init: function () {
		this.command = document.getElementById("category-command");
		this.panel = document.getElementById("category-view");
		this.tree = document.getElementById("category-tree");
		},

	select: function () {
		alert("Category.select");
		}

	}


Crypto = {
	b64: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",
	fail: 0,
	key: null,

	login: null,

	// check existence of hash number h
	hasHash: function (h) {
		return (Journals.getProperty("hash" + h) != "");
		},

	init: function () {
		this.login = document.getElementById("login-command");
		this.login.setAttribute("disabled", !this.hasHash(1));
		},

	// set hash number h to value v
	setHash: function (h, v) {
		Journals.setProperty("hash" + h, v);
		},

	sha1: function (s, g) {
		const p = 63, f = 255, m = 0xFFFF, a2 = 24, a5 = 192;

		if (!g) g = (Math.random() * m * m) & -1;
		s += String.fromCharCode(g); 

		var i, j;
		var x = [];
		var n = s.length << 4;
		for (i = 0; i < n; i += 16) x[i >>> 5] |= (s.charCodeAt(i >>> 4) & m) << (24 - i & 31);
		x[n >>> 5] |= 0x80 << (24 - n & 31);
		x[((n + 64 >>> 9) << 4) + 15] = n;
		for (j = 0; j < x.length; ++j) if (x[j] == undefined) x[j] = 0;

		var w = new Array(80);
		var a =  1732584193, b = -271733879, c = -1732584194, d = 271733878, e = -1009589776;
		for (i = 0; i < x.length; i += 16) {
			var a0 = a, b0 = b, c0 = c, d0 = d, e0 = e;
			for (j = 0; j < 80; ++j) {
				w[j] =  (j < 16)? x[i + j] : rol(w[j - 3] ^ w[j - 8] ^ w[j - 14] ^ w[j - 16], 1);
				var t = this._add(
					this._add(rol(a, 5), (j < 20)? (b & c) | ((~b) & d) : (j < 40)? b ^ c ^ d : (j < 60)? (b & c) | (b & d) | (c & d) : b ^ c ^ d),
					this._add(this._add(e, w[j]), (j < 20) ?  1518500249 : (j < 40) ?  1859775393 : (j < 60) ? -1894007588 : -899497514)
					);
				e = d, d = c,	c = rol(b, 30),	b = a, a = t;
				}
			a = this._add(a, a0),	b = this._add(b, b0),	c = this._add(c, c0),	d = this._add(d, d0),	e = this._add(e, e0);
			}

		var r = "";
		var y = [a, b, c, d, e, g];
		for (i = 0; i < a2; i += 3) {
			t = (((y[i >> 2] >> 8 * (3 - i & 3)) & f) << 16)
				| (((y[i + 1 >> 2] >> 8 * (3 - (i + 1) & 3)) & f) << 8 )
				| ((y[i + 2 >> 2] >> 8 * (3 - (i + 2) & 3)) & f);
			for (j = 0; j < 4; j++)
				r += (i * 8 + j * 6 > a5)? "=" : this.b64.charAt((t >> 6 * (3 - j)) & p);
			}
		return r;
		},

	// validate p from hash number h
	validate: function (p, h) {
		var h = Journals.getProperty("hash" + h);
		if (h == "") return false;
		else return (this.sha1(p, this._mine(h)) == h);
		},

	_add: function (x, y) {
		const f = 0xFFFF;
		var l = (x & f) + (y & f);
		return (((x >>> 16) + (y >>> 16) + (l >>> 16)) << 16) | (l & f);
		},

	// extract salt
	_mine: function (r) {
		var s = 0;
		for (i = 0; i < 6; ++i) s |= this.b64.indexOf(r.charAt(31 - i)) << 6 * i;
		return s & -1;
		}

	}


Diary = {
	command: null,
	month: null,
	panel: null,
	picker: null,
	tree: null,
	year:null,

	display: function () {
		this.tree.database.AddDataSource(Journal.ds);
		this.tree.builder.rebuild();
		Sidebar.display(this.panel);
		View.set("view-diary");
		},

	enable: function (enabled) {
		with (this.command) 
			if (enabled) removeAttribute("disabled")
			else setAttribute("disabled", "true");
		},

	init: function () {
		this.command = document.getElementById("diary-command");
		this.panel = document.getElementById("diary-view");
		this.picker = document.getElementById("diary-month-picker");
		this.tree = document.getElementById("diary-tree");
		this.picker.owner = this;
		this.picker.changeMonth(0);
		},

	// called by picker with changed Date
	month: function (d) {
		if (Schedule.ds) this.tree.database.RemoveDataSource(Schedule.ds);
		Schedule.month(d.getFullYear(), d.getMonth());
		this.tree.database.AddDataSource(Schedule.ds);
		this.tree.builder.rebuild();
		}

	}


Entry = {
	node: null,
	editmode: false,
	isnew: false,
	broadcaster: null,

	get name () {
		return this._load("name");
		},

	get category () {
		return this._load("category");
		},

	get index () {
		return this._load("index");
		},

	get classes () {
		return this._load("class");
		},

	get created () {
		return this._load("created");
		},

	get modified () {
		return this._load("modified");
		},

	get start () {
		return this._load("start");
		},

	get end () {
		return this._load("end");
		},

	get text () {
		return decodeURIComponent(this._load("text"));
		},

	set name (v) {
		this._save("name", v);
		Subject.refresh();
		},

	set category (v) {
		this._save("category", v);
		},

	set index (v) {
		this._save("index", v);
		},

	set classes (v) {
		this._save("class", v);
		},

	set created (v) {
		this._save("created", v);
		},

	set modified (v) {
		this._save("modified", v);
		},

	set start (v) {
		this._save("start", v);
		},

	set end (v) {
		this._save("end", v);
		},

	set text (v) {
		this._save("text", v);
		},

	// initialize from rdf node id
	init: function (n) {
		if (!this.broadcaster) this.broadcaster = document.getElementById("entry-broadcaster");
		this.broadcaster.setAttribute("disabled", "true");

		Body.release();
		try {
			with (this) {
				if (n) {
					node = RDF.GetResource(n);
					editmode = false;
					isnew = false;
					}
				else {
					node = Journal.createNode();
					name = "";
					category = "";
					index = "";
					classes = "";
					created = "";
					modified = "";
					start = "";
					end = "";
					text = "";
					editmode = false;
					isnew = true;
					}
				}
			Body.display(true);
			this.broadcaster.removeAttribute("disabled");
			if (this.isnew) setTimeout("Body.mode(true);", 1);
			}
		catch (e) {alert("Entry.init " + e.lineNumber + "\n" + e);}
		},

	release: function () {
		if (!this.node) return;
		this.node = null;
		this.broadcaster.setAttribute("disabled", "true");
		},

	remove: function () {
		if (!this.node) return;
		Journal.deleteNode(this.node);
		this.node = null;
		},

	_load: function (property) {
		var r = Journal.ds.GetTarget(this.node, Journals.resource(property), true);
		if (r)
			return r.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
		else
			return "";
		},

	_save: function (property, val) {
		try {
			var ds = Journal.ds;
			var p = Journals.resource(property);
			var r = ds.GetTarget(this.node, p, true);
			if (r) ds.Unassert(this.node, p, r);
			ds.Assert(this.node, p, RDF.GetLiteral(val), true);
			}
		catch (e) {alert("Entry._save\n" + e);}
		}

	}


Error = {
	alert: function (m, e) {
		window.alert("ERROR " + m + "\n" + e);
		}	
	}


Help = {
	features: "chrome,all,alwaysRaised,dialog=no",
	home:     "welcome",
	rdf:      "chrome://journal/locale/help/help.rdf",
	xul:      "chrome://journal/content/help/help.xul",

	open: function (iscontext) {

try {
    const p = Components
			.classes["@mozilla.org/embedcomp/dialogparam;1"]
			.createInstance(Components.interfaces.nsIDialogParamBlock);
    p.SetNumberStrings(2);
    p.SetString(0, this.rdf);
    p.SetString(1, this.home);

		const ww = Components
			.classes["@mozilla.org/embedcomp/window-watcher;1"]
			.getService(Components.interfaces.nsIWindowWatcher);

		ww.openWindow(null, this.xul, "_blank", this.features, p);
} catch (e) {alert("Help.open\n" + e);}
		}
	}


Index = {
	command: null,
	panel: null,
	tree: null,

	display: function () {
		this.tree.datasources = Journal.ds.URI;
		Sidebar.display(this.panel);
		View.set("view-index");
		},

	enable: function (enabled) {
		with (this.command) 
			if (enabled) removeAttribute("disabled")
			else setAttribute("disabled", "true");
		},

	init: function () {
		this.command = document.getElementById("index-command");
		this.panel = document.getElementById("index-view");
		this.tree = document.getElementById("index-tree");
		},

	select: function () {
		}

	}


Journal = {
	ds: null,
	entries: null,
	root: null,
	broadcaster: null,

	createNode: function () {
		var ok = false;
		while (!ok) {
			var d = Math.floor((Math.random() * 0x100000000)).toString(16);
			var r = RDF.GetResource("urn:entry:" + ("0000000" + d).substr(d.length - 1));
			ok = this.entries.IndexOf(r) < 0;
			}
			this.entries.AppendElement(r);
			return r;
		},

	deleteNode: function (n) {
		var d = this.ds;
		var a = d.ArcLabelsOut(n);
		while (a.hasMoreElements()) {
			var p = a.getNext();
			d.Unassert(n, p, d.GetTarget(n, p, true));
			}
		this.entries.RemoveElement(n, true);
		},

	getProperty: function (p) {
		try {
			var r = this.ds
				.GetTarget(this.root, RDF.GetResource(Journals.jns + p), true);
			if (!r) return "";
			return r
				.QueryInterface(Components.interfaces.nsIRDFLiteral)
				.Value;
			}
		catch (e) {alert("Journal.getProperty " + p + "\n" + e);}
		},

	init: function (filename) {
		if (!this.broadcaster) this.broadcaster = document.getElementById("journal-broadcaster");
		this.broadcaster.setAttribute("disabled", "true");

		var jf = Pref.directory;
		jf.append(filename + ".rdf");
		if (!jf.exists()) {
			alert("Can't find journal file " + filename + ".rdf\n" + e);
			return;
			}

		var uri = Components
			.classes["@mozilla.org/network/protocol;1?name=file"]
			.createInstance(Components.interfaces.nsIFileProtocolHandler)
			.newFileURI(jf);

		try {
			this.ds = RDF.GetDataSourceBlocking(uri.spec);
			}
		catch (e) {
			alert("Can't read journal file " + filename + ".rdf\n" + e);
			return;
			}

		this.root = RDF.GetResource("urn:journal");
		this.entries = RCU.MakeSeq(this.ds, RDF.GetResource("urn:journal:entries"));
		this.broadcaster.removeAttribute("disabled");
		},

	setProperty: function (p, v) {
		try {
			p = RDF.GetResource(Journals.jns + p);
			var r = this.ds.GetTarget(this.root, p, true);
			if (r) this.ds.Unassert(this.root, p, r);
			this.ds.Assert(this.root, p, RDF.GetLiteral(v), true);
			}
		catch (e) {alert("Journal.setProperty " + p + "\n" + e);}
		}

	}


Journals = {
	ds: null,
	jns: "http://journal.mozdev.org/ns#",

	root: null,
	jnsDesc: null,
	jnsFile: null,
	jnsJournal: null,
	jnsList: null,
	jnsName: null,

	list: null,
	panel: null,
	seq: null,

	// add a journal to the list
	add: function (name, file, desc) {
		var anon = RDF.GetAnonymousResource();
		this.ds.Assert(anon, this.jnsName, RDF.GetLiteral(name), true);
		this.ds.Assert(anon, this.jnsFile, RDF.GetLiteral(file), true);
		this.ds.Assert(anon, this.jnsDesc, RDF.GetLiteral(desc), true);
		this.seq.AppendElement(anon);
		this._sort();
		},

	// null: user pressed the new journal button, else create preset
	create: function (preset) {
		var ds = Components
			.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
			.createInstance(Components.interfaces.nsIRDFDataSource);
		var ok = (preset)? this._createPreset(ds, preset) : User.properties(ds);
		if (!ok) return;

		var n = ds
			.GetTarget(
				RDF.GetResource("urn:journal"),
				RDF.GetResource(Journals.jns + "name"),
				true)
			.QueryInterface(Components.interfaces.nsIRDFLiteral)
			.Value;

		var jf = Pref.directory;
		jf.append(n.replace(/\s/g, "") + ".rdf");
		jf.createUnique(0, 292)

		var js = Components
			.classes["@mozilla.org/network/file-output-stream;1"]
			.createInstance(Components.interfaces.nsIFileOutputStream);
		js.init(jf, 0x24, 292, null);

		var rs = Components
			.classes["@mozilla.org/rdf/xml-serializer;1"]
			.createInstance(Components.interfaces.nsIRDFXMLSerializer);
		rs.init(ds);
		rs.QueryInterface(Components.interfaces.nsIRDFXMLSource).Serialize(js);

		js.close();

		var nf = jf.leafName.replace(/.rdf$/, "");

		var d = ds
			.GetTarget(
				RDF.GetResource("urn:journal"),
				RDF.GetResource(Journals.jns + "description"),
				true)
			.QueryInterface(Components.interfaces.nsIRDFLiteral)
			.Value;

		this.add(n, nf, d);

		for (var i = 0; i < this.list.getRowCount(); ++i) {
			var ji = this.list. getItemAtIndex(i);
			if (ji.value == nf) {
				this.list.selectItem(ji);
				break;
				}
			}

		},

	get count () {
		return this.seq.GetCount();
		},

	getProperty: function (p) {
		try {
			var r = this.ds
				.GetTarget(this.root, RDF.GetResource(this.jns + p), true);
			if (!r) return "";
			return r
				.QueryInterface(Components.interfaces.nsIRDFLiteral)
				.Value;
			}
		catch (e) {alert("Journals.getProperty " + p + "\n" + e);}
		},

	display: function () {
		Sidebar.display(this.panel);
		View.set("view-journals");
		},

	init: function () {
		try {
			this.panel = document.getElementById("journals-view");

			this.root = RDF.GetResource("urn:journals:root");
			this.jnsDesc = RDF.GetResource(this.jns + "description");
			this.jnsFile = RDF.GetResource(this.jns + "file");
			this.jnsJournal = RDF.GetResource(this.jns + "journal");
			this.jnsList = RDF.GetResource(this.jns + "list");
			this.jnsName = RDF.GetResource(this.jns + "name");

			var jf = Pref.directory;
			jf.append("journals.rdf");
			if (!jf.exists()) {
				this._createList(jf);
				User.wizard();
				}

			var uri = Components
				.classes["@mozilla.org/network/protocol;1?name=file"]
				.createInstance(Components.interfaces.nsIFileProtocolHandler)
				.newFileURI(jf);

			try {
				this.ds = RDF.GetDataSourceBlocking(uri.spec);
				var list = this.ds.GetTarget(this.root, this.jnsList, true);
				this.seq = RCU.MakeSeq(this.ds, list);
				}
			catch (e) {
				alert("Can't read master journals list\n" + e);
				return;
				}

			this.list = document.getElementById("journals-listbox");
			this.list.datasources = this.ds.URI;
			this._sort();
			if (this.count > 0) this.list.selectedIndex = 0;
			}
		catch (e) {alert("Journals.init " + e.lineNumber + "\n" + e);}
		},

	// remove selected journal
	remove: function () {
		if (!this.list.selectedItem) return;

		var filetoo = {value: false};
		var ok = User.prompter.confirmCheck(
			self,
			this.list.getAttribute("deleteTitle"),
			this.list.getAttribute("deletePrompt").replace(/%/, this.list.selectedItem.label),
			this.list.getAttribute("deleteCheck"),
			filetoo);
		if (!ok) return;

		document.title = App.title;
		Body.display(null);

		var sv = this.list.selectedItem.value;
		var s = this.ds.GetSource(
			this.jnsFile,
			RDF.GetLiteral(sv),
			true);
		var sa = this.ds.ArcLabelsOut(s);
		while (sa.hasMoreElements()) {
			var sp = sa.getNext();
			this.ds.Unassert(s, sp, this.ds.GetTarget(s, sp, true));
			}
		this.seq.RemoveElement(s, true);
		if (!filetoo.value) return;

		var jf = Pref.directory;
		var jf = Pref.directory;
		jf.append(sv + ".rdf");

		try {jf.remove(false);}
		catch (e) {alert("Journals.remove " + sv +".rdf\n" + e);}
		},

	// rdf resource from property name
	resource: function (name) {
		return RDF.GetResource(this.jns + name);
		},

	// user selected a journal
	select: function (event) {
		try {
			if (!this.list.selectedItem) return;
			Journal.init(this.list.selectedItem.value);
			document.title = this.list.selectedItem.label + " - " + App.title;
			Category.enable(true);
			Diary.enable(true);
			Index.enable(true);
			Subject.enable(true);
			Body.display(null);
			}
		catch (e) {alert("Journals.select " + e.lineNumber + "\n" + e);}
		},

	setProperty: function (p, v) {
		try {
			p = RDF.GetResource(this.jns + p);
			var r = this.ds.GetTarget(this.root, p, true);
			if (r) this.ds.Unassert(this.root, p, r);
			this.ds.Assert(this.root, p, RDF.GetLiteral(v), true);
			}
		catch (e) {alert("Journals.setProperty " + p + "\n" + e);}
		},

	// callback from setup wizard
	setup: function (n, jn, jd) {
		switch (n) {
			case 0: // user name
				Journals.setProperty("owner", jn);
				break;
			case 1: // diary
			case 2: // notes
			case 3: // quotes
				Journals.create({type: n, name: jn, description: jd});
				break;
			}
		},

	// user changed journal properties
	update: function () {
		var sv = this.list.selectedItem.value;
		var s = this.ds.GetSource(
			this.jnsFile,
			RDF.GetLiteral(sv),
			true);

		var ok = true;

		var n = this.ds
			.GetTarget(s, this.jnsName, true)
			.QueryInterface(Components.interfaces.nsIRDFLiteral);
		var jn = Journal.getProperty("name");
		if (n.Value != jn) {
			this.ds.Change(s, this.jnsName, n, RDF.GetLiteral(jn));
			ok = false;
			}

		var d = this.ds
			.GetTarget(s, this.jnsDesc, true)
			.QueryInterface(Components.interfaces.nsIRDFLiteral);
		var jd = Journal
			.getProperty("description")
			.match(/(^[^\n]*)/)[0];
		if (d.Value != jd) {
			this.ds.Change(s, this.jnsDesc, d, RDF.GetLiteral(jd));
			ok = false;
			}

		if (!ok) {
			var ss = Components
				.classes["@mozilla.org/xul/xul-sort-service;1"]
				.getService(Components.interfaces.nsIXULSortService);
			ss.sort(this.list, "http://journal.mozdev.org/ns#name", "ascending");
			this.list.value = sv;
			}

		},


	// create an initial journals list from scratch
	_createList: function(f) {
		try {
			f.create(0, 420);

			var s = Components
				.classes["@mozilla.org/network/file-output-stream;1"]
				.createInstance(Components.interfaces.nsIFileOutputStream);
			s.init(f, 0x24, 292, null);

			var head = '<?xml version="1.0"?>'
				+ '\n<!DOCTYPE RDF:RDF>'
				+ '\n<RDF:RDF'
				+ '\n\txmlns:RDF="http://www.w3.org/1999/02/22-rdf-syntax-ns#"'
				+ '\n\txmlns:JL="' + this.jns + '">'
				+ '\n<RDF:Description RDF:about="urn:journals:root">'
				+ '\n\t<JL:list>'
				+ '\n\t\t<RDF:Seq>'
				+ '\n\t\t</RDF:Seq>'
				+ '\n\t</JL:list>'
				+ '\n</RDF:Description>'
				+ '\n</RDF:RDF>';

			s.write(head, head.length);
			s.close();
			}
		catch (e) {
			Error.alert("Can't create file " + f.path, e);
			}
		},

	// create journal from preset object
	_createPreset: function (ds, preset) {
		var r = RDF.GetResource("urn:journal");
		var p = (preset.type == 2)? "rgb(220,233,221);34,22,88" : "";
		ds.Assert(
			r,
			RDF.GetResource(Journals.jns + "preset"),
			RDF.GetLiteral(preset.type),
			true);
		ds.Assert(
			r,
			RDF.GetResource(Journals.jns + "name"),
			RDF.GetLiteral(preset.name),
			true);
		ds.Assert(
			r,
			RDF.GetResource(Journals.jns + "description"),
			RDF.GetLiteral(preset.description),
			true);
		ds.Assert(
			r,
			RDF.GetResource(Journals.jns + "paper"),
			RDF.GetLiteral(p),
			true);
		return true;
		},

	_sort: function () {
		var ss = Components
			.classes["@mozilla.org/xul/xul-sort-service;1"]
			.getService(Components.interfaces.nsIXULSortService);
		ss.sort(this.list, "http://journal.mozdev.org/ns#name", "ascending");
		}

	}


Pref = {
	branch: Components
		.classes['@mozilla.org/preferences-service;1']
		.getService(Components.interfaces.nsIPrefService)
		.getBranch("journal."),

	get directory () {
		var n = "";
		try {n = branch.getCharPref("directory");} catch (e) {}

		if (n != "") {
			try {
				return Components
					.classes["@mozilla.org/file/local;1"]
					.getService(Components.interfaces.nsILocalFile)
					.initWithPath(n);
				}
			catch (e) {
				Error.alert("Can't use data directory " + n, e);
				}
			}

		var d = Components
			.classes["@mozilla.org/file/directory_service;1"]
			.getService(Components.interfaces.nsIProperties)
			.get("AppRegD", Components.interfaces.nsIFile);
		d.append("Journals");
		return d;
		}
	}


RCU = Components
	.classes['@mozilla.org/rdf/container-utils;1']
	.getService(Components.interfaces.nsIRDFContainerUtils);


RDF = Components
	.classes['@mozilla.org/rdf/rdf-service;1']
	.getService(Components.interfaces.nsIRDFService);


Schedule = {
	ds: null,

	init: function () {
		},

	// build data for year y, month m (0-11)
	month: function (y, m) {
		this.ds = Components
			.classes['@mozilla.org/rdf/datasource;1?name=in-memory-datasource']
			.createInstance(Components.interfaces.nsIRDFDataSource);
		var jn = RDF.GetResource("http://journal.mozdev.org/ns#name");
		var m1 = m + 1;
		var p = "urn:diary:" + y + ((m1 < 10)? "0" + m1 : "" + m1);

		/** obsolete?
		// check this month is not already built
		var li = RDF.GetResource(p + "01");
		if (this.ds.GetTarget(li, jn, true)) return;
		 **/

		var now = new Date();
		now.setYear(y);
		now.setMonth(m);
		now.setDate(1);

		var seq = RCU.MakeSeq(this.ds, RDF.GetResource("urn:journal:diary:dates"));
		while (now.getMonth() == m) {
			var d = now.getDate();
			li = RDF.GetResource(p + ((d < 10)? "0" + d : "" + d));
			seq.AppendElement(li);
			var w = Calendar.dayShort[now.getDay()];
			var dn = (d < 10)? "   " + d + " " + w : d + " " + w;
			this.ds.Assert(li, jn, RDF.GetLiteral(dn), true);
			now.setDate(d + 1);
			}

		}

	}


/** schedule ==================================================================
<RDF:Seq about="urn:journal:diary:dates">
	<RDF:li><RDF:Description about="urn:diary:20041101" JL:name="  1 Mon"/></RDF:li>
	<RDF:li><RDF:Description about="urn:diary:20041102" JL:name="  2 Tue"/></RDF:li>
	<RDF:li><RDF:Description about="urn:diary:20041103" JL:name="  3 Wed"/></RDF:li>
	<RDF:li><RDF:Description about="urn:diary:20041104" JL:name="  4 Thu"/></RDF:li>
	<RDF:li><RDF:Description about="urn:diary:20041105" JL:name="  5 Fri"/></RDF:li>
	<RDF:li><RDF:Description about="urn:diary:20041106" JL:name="  6 Sat"/></RDF:li>
	<RDF:li><RDF:Description about="urn:diary:20041107" JL:name="17 Sun"/></RDF:li>
</RDF:Seq>

for each day...
	for each scheduled category in Journal...
		if Journal.ds has no real entry for this day and category...
			and if category's schedule wants an entry on this day...
				make an entry in Schedule.ds


=============================================================================== **/


Sidebar = {
	deck: null,

	display: function (panel) {
		this.deck.selectedPanel = panel;
		this.showHide(true);
		},

	init: function () {
		this.deck = document.getElementById("view-deck");
		},

	showHide: function (seen) {
		with (document.getElementById("sidebar")) {
			if (seen == !collapsed) return;
			if (seen == undefined) seen = collapsed;
			collapsed = !seen;
			}
		document.getElementById("sidebar-splitter")
			.setAttribute("state", ((seen)? "" : "collapsed"));
		document.getElementById("view-sidebar").setAttribute("checked", seen);
		document.getElementById("tool-sidebar").checked = !seen;
		},

	resized: function () {
		var c = document
			.getElementById("sidebar")
			.collapsed;
		document.getElementById("view-sidebar").setAttribute("checked", !c);
		document.getElementById("tool-sidebar").checked = c;
		}

	}



Status = {
	panel: null,

	set message (m) {
		this.panel.label = m;
		setTimeout("Status.clear();", 2000);
		},

	clear: function () {
		this.panel.label = "";
		},

	init: function () {
		this.panel = document.getElementById("task-status");
		}

	}


Subject = {
	command: null,
	list: null,
	panel: null,

	display: function () {
		this.list.datasources = Journal.ds.URI;
		Sidebar.display(this.panel);
		View.set("view-subject");
		},

	enable: function (enabled) {
		with (this.command) 
			if (enabled) removeAttribute("disabled")
			else setAttribute("disabled", "true");
		},

	init: function () {
		this.command = document.getElementById("subject-command");
		this.list = document.getElementById("subject-listbox");
		this.panel = document.getElementById("subject-view");
		},

	refresh: function () {
		var s = Components
			.classes["@mozilla.org/xul/xul-sort-service;1"]
			.getService(Components.interfaces.nsIXULSortService);
		s.sort(this.list, "http://journal.mozdev.org/ns#name", "ascending");
		},

	// user selected an entry (or something else happened)
	select: function () {
		if (this.list.selectedItem) Entry.init(this.list.selectedItem.id);
		}
	}



User = {
	prompter: null,
	watcher: null,

	about: function (show) {
		with (document.getElementById("splitter-aboutfix")) {
			setAttribute("aboutfix", show);
			}
		with (document.getElementById("about-box")) {
			collapsed = !show;
			if (show) {
				style.left = (window.innerWidth - boxObject.width) / 2 + "px";
				style.top = (window.innerHeight - boxObject.height) / 2 + "px";
				}
			}
		if (show)	{
			window.addEventListener("click", this, true);
			with (document.getElementById("about-text")) {
				value = navigator.userAgent + " Journal 0.1";
				focus();
				}
			}
		else {
			window.removeEventListener("click", this, true);
			document.getElementById("about-text").blur();
			}
		},

	aboutclick: function (event) {
		if (event.target.localName == "vbox") this.about(false);
		else event.stopPropagation();
		},

	aboutkey: function (event) {
		var k = event.charCode || event.keyCode;
		if (k == event.DOM_VK_ESCAPE
			|| k == event.DOM_VK_RETURN
			|| k == event.DOM_VK_ENTER
			|| k == event.DOM_VK_SPACE)
			User.about(false);
		event.stopPropagation();
		},

	alert: function (msg) {
		this.prompter.alert(self, document.documentElement.getAttribute("error"), msg);
		},

	// delete a journal or entry
	deleteSomething: function () {
		var f = document.commandDispatcher.focusedElement;
		if (!f) return;
		if (f.id && f.id == "journals-listbox") Journals.remove();
		else Body.remove();
		},

	// hide About box
	handleEvent: function (event) {
		var id = event.target.id || "";
		if (id != "help-about" && id != "about-text") User.about(false);
		},

	init: function () {
		this.watcher = Components
			.classes["@mozilla.org/embedcomp/window-watcher;1"]
			.getService(Components.interfaces.nsIWindowWatcher);
		this.prompter = Components
				.classes["@mozilla.org/embedcomp/prompt-service;1"]
				.getService(Components.interfaces.nsIPromptService);
		},

	login: function () {
		window.login = {};
		window.login.user = "Roderick";	// get this from Journals
		window.login.validate = Crypto.validate;
		window.login.fail = Crypto.fail;	// get this from Journals?

		this.watcher.openWindow(
			self,
			"chrome://journal/content/login/login.xul",
			"journal:login",
			"modal,chrome,dialog,centerscreen,resizable=no",
			null);

		Crypto.fail = window.login.fail;
		window.login = null;
		},

	pref: function () {
		window.login = {};
		window.login.hasHash = Crypto.hasHash;
		this.watcher.openWindow(
			self,
			"chrome://journal/content/pref/pref.xul",
			"journal:preferences",
			"chrome,dialog,centerscreen,resizable=no",
			null);
		},

	properties: function (ds) {
		var isnew = true;
		if (!ds) {
			ds = Journal.ds;
			isnew = false;
			}
		this.watcher.openWindow(
			self,
			"chrome://journal/content/prop/prop.xul",
			"journal:properties",
			"modal,chrome,dialog,centerscreen,resizable",
			ds);
		var dr = RDF.GetResource("urn:dialog");
		var dp = RDF.GetResource("urn:dialog:ok");
		var dt = RDF.GetLiteral("true");
		var ok = ds.HasAssertion(dr, dp, dt, true);
		if (ok) {
			ds.Unassert(dr, dp, dt, true);
			if (isnew) return true;
			Journals.update();
			Body.redisplay();
			}
		return ok;
		},

	wizard: function () {
		window.login = {};
		window.login.setup = Journals.setup;
		this.watcher.openWindow(
			self,
			"chrome://journal/content/wizard/wizard.xul",
			"journal:wizard",
			"chrome,dialog,centerscreen",
			null);
		},

	ztest: function () {
		try {
			var w = document.commandDispatcher.focusedWindow;
/*			var f = w.document.commandDispatcher.focusedElement; */
			alert(w.document);
/**
			alert("window " + w +
				"\nfocus " + f + 
				"\ntag " + ((f)? f.localName : "") + 
				"\nid " + ((f)? f.id : ""));
 **/
			}
		catch (e) {alert("User.ztest\n" + e);}
		}

	}


View = {
	menu0: null,	
	tool0: null,

	init: function () {
		this.menu0 = document.getElementById("view-journals");
		this.tool0 = document.getElementById("tool-journals");
		},

	set: function (menuid) {
		try {
			var m = this.menu0;
			var t = this.tool0;
			for (var i = 0; i < 5; ++i) {
				if (m.id == menuid) {
					m.setAttribute("checked", "true");
					t.checked = true;
					}
				else {
					m.removeAttribute("checked");
					t.checked = false;
					}
				m = m.nextSibling;
				while (m.localName != "menuitem") m = m.nextSibling;
				t = t.nextSibling;
				}
			}
		catch (e) {
			alert("View.set\n" + e);
			}
		}
	
	}
